==> gradle-enterprise-database-setup-zip-2024.3.zip: sql/create_db_users.sql <==
-- The create_db_objects.sql script must be run before this one.

-- To alter the specific db users set up, edit the final DO block in the file.

-- Does NOT attempt to create credentials for the created users - this is left to customers to do

CREATE PROCEDURE pg_temp.verify_required_roles() AS $proc$
DECLARE
  app TEXT;
  role TEXT;
BEGIN
  RAISE NOTICE '---';
  RAISE NOTICE 'Verifying required roles';
  FOREACH app in ARRAY ARRAY['admin', 'build_cache', 'build_scans', 'keycloak'] LOOP
    FOREACH role in ARRAY ARRAY['owner', 'user'] LOOP
      IF NOT EXISTS (SELECT FROM pg_roles WHERE rolname = app || '_' || role) THEN
        RAISE EXCEPTION 'Expected role %_% to exist, but it did not. Please run the create_db_objects.sql script first', app, role;
      END IF;
    END LOOP;
  END LOOP;
END;
$proc$ LANGUAGE plpgsql;

CREATE PROCEDURE pg_temp.grant_role_to_user(rolename TEXT, username TEXT) AS $proc$
DECLARE
  r TEXT;
BEGIN
  IF NOT EXISTS (SELECT FROM pg_auth_members WHERE roleid = rolename::regrole AND member = username::regrole) THEN
    EXECUTE format('GRANT %I TO %I', quote_ident(rolename), quote_ident(username));
    RAISE NOTICE 'Granted % to %', rolename, username;
  ELSE
    RAISE NOTICE 'User % already has role %', username, rolename;
  END IF;
END;
$proc$
LANGUAGE plpgsql;

CREATE PROCEDURE pg_temp.make_user(username TEXT) AS $proc$
DECLARE
  acting_user TEXT = (SELECT current_user);
  postgres_server_version_num NUMERIC;
  was_granted BOOLEAN;
BEGIN
  IF NOT EXISTS (SELECT FROM pg_roles WHERE rolname = username) THEN
    BEGIN
      EXECUTE format('CREATE USER %I', username);
      RAISE NOTICE 'Created user %', username;
    EXCEPTION WHEN duplicate_object THEN
      RAISE DEBUG 'User % already exists', username;
    END;
  ELSE
    RAISE NOTICE 'User % already exists', username;
  END IF;

  -- in pg16+ the acting user needs the admin option on the user roles in order to set their password in Azure environments
  SELECT version FROM current_setting('server_version_num') as version INTO postgres_server_version_num;
  IF postgres_server_version_num >= 160000 THEN
    SELECT EXISTS (
      SELECT
      FROM pg_auth_members
      WHERE member::regrole = acting_user::regrole
        AND roleid::regrole = username::regrole
        AND admin_option
    ) INTO was_granted;
    IF NOT was_granted THEN
      EXECUTE format('GRANT %I TO %I WITH ADMIN TRUE', username, acting_user);
    END IF;
  END IF;
END;
$proc$
LANGUAGE plpgsql;

CREATE PROCEDURE pg_temp.setup_user(username TEXT, suffix TEXT, description TEXT) AS $proc$
DECLARE
  app TEXT;
BEGIN
  RAISE NOTICE '---';
  RAISE NOTICE 'Setting up % user %', description, username;
  CALL pg_temp.make_user(username);
  FOREACH app in ARRAY ARRAY['admin', 'build_cache', 'build_scans', 'keycloak'] LOOP
    CALL pg_temp.grant_role_to_user(app || '_' || suffix, username);
  END LOOP;
END;
$proc$ LANGUAGE plpgsql;

CREATE PROCEDURE pg_temp.setup_app_user(username TEXT) AS $proc$
BEGIN
  CALL pg_temp.setup_user(username, 'user', 'application');
END;
$proc$ LANGUAGE plpgsql;

CREATE PROCEDURE pg_temp.setup_migrator_user(username TEXT) AS $proc$
BEGIN
  CALL pg_temp.setup_user(username, 'owner', 'migrator');
END;
$proc$ LANGUAGE plpgsql;

CREATE PROCEDURE pg_temp.setup_monitor_user(username TEXT) AS $proc$
BEGIN
  RAISE NOTICE '---';
  RAISE NOTICE 'Setting up monitor user %', username;
  CALL pg_temp.make_user(username);
  CALL pg_temp.grant_role_to_user('pg_read_all_stats', username);
END;
$proc$ LANGUAGE plpgsql;

DO $$
BEGIN
  CALL pg_temp.verify_required_roles();

  -- EDIT THE FOLLOWING LINES TO ALTER DB USERS SET UP
  CALL pg_temp.setup_app_user('ge_app');
  CALL pg_temp.setup_migrator_user('ge_migrator');
  CALL pg_temp.setup_monitor_user('ge_monitor');
  -- END - DO NOT EDIT PAST HERE

  COMMIT;
  RAISE NOTICE '---';
  RAISE NOTICE 'Database users set up successfully';
END;
$$ LANGUAGE plpgsql;

==> gradle-enterprise-database-setup-zip-2025.3.zip: sql/create_db_users.sql <==
-- The create_db_objects.sql script must be run before this one.

-- To alter the specific db users set up, edit the final DO block in the file.

-- Does NOT attempt to create credentials for the created users - this is left to customers to do

CREATE PROCEDURE pg_temp.verify_required_roles() AS $proc$
DECLARE
  app TEXT;
  role TEXT;
BEGIN
  RAISE NOTICE '---';
  RAISE NOTICE 'Verifying required roles';
  FOREACH app in ARRAY ARRAY['admin', 'build_cache', 'build_scans', 'keycloak'] LOOP
    FOREACH role in ARRAY ARRAY['owner', 'user'] LOOP
      IF NOT EXISTS (SELECT FROM pg_roles WHERE rolname = app || '_' || role) THEN
        RAISE EXCEPTION 'Expected role %_% to exist, but it did not. Please run the create_db_objects.sql script first', app, role;
      END IF;
    END LOOP;
  END LOOP;
END;
$proc$ LANGUAGE plpgsql;

CREATE PROCEDURE pg_temp.grant_role_to_user(rolename TEXT, username TEXT) AS $proc$
DECLARE
  r TEXT;
BEGIN
  IF NOT EXISTS (SELECT FROM pg_auth_members WHERE roleid = rolename::regrole AND member = username::regrole) THEN
    EXECUTE format('GRANT %I TO %I', quote_ident(rolename), quote_ident(username));
    RAISE NOTICE 'Granted % to %', rolename, username;
  ELSE
    RAISE NOTICE 'User % already has role %', username, rolename;
  END IF;
END;
$proc$
LANGUAGE plpgsql;

CREATE PROCEDURE pg_temp.make_user(username TEXT) AS $proc$
DECLARE
  acting_user TEXT = (SELECT current_user);
  postgres_server_version_num NUMERIC;
  was_granted BOOLEAN;
BEGIN
  IF NOT EXISTS (SELECT FROM pg_roles WHERE rolname = username) THEN
    BEGIN
      EXECUTE format('CREATE USER %I', username);
      RAISE NOTICE 'Created user %', username;
    EXCEPTION WHEN duplicate_object THEN
      RAISE DEBUG 'User % already exists', username;
    END;
  ELSE
    RAISE NOTICE 'User % already exists', username;
  END IF;

  -- in pg16+ the acting user needs the admin option on the user roles in order to set their password in Azure environments
  SELECT version FROM current_setting('server_version_num') as version INTO postgres_server_version_num;
  IF postgres_server_version_num >= 160000 THEN
    SELECT EXISTS (
      SELECT
      FROM pg_auth_members
      WHERE member::regrole = acting_user::regrole
        AND roleid::regrole = username::regrole
        AND admin_option
    ) INTO was_granted;
    IF NOT was_granted THEN
      EXECUTE format('GRANT %I TO %I WITH ADMIN TRUE', username, acting_user);
    END IF;
  END IF;
END;
$proc$
LANGUAGE plpgsql;

CREATE PROCEDURE pg_temp.revoke_user_from_current(username TEXT) AS $proc$
DECLARE
  was_granted BOOLEAN;
BEGIN
  -- In 2024.3 we granted this permission without revoking it after it was needed.
  -- To fix up previously broken installations, we now revoke it unconditionally.
  -- It's very unlikely to be needed, and caused issues when using RDS+IAM where the
  -- transitive role grant to the rds_iam role would then break password authentication
  -- for the rds superuser.
  -- At some point in the future when we are unlikely to have older installs to fix up
  -- we may decide to do what we did with similar role grants in create_db_objects.sql
  -- and only revoke the grant if we just did it ourselves.

  -- we still check for the role grant first to not spam logs with warnings
  SELECT EXISTS (
    SELECT
      FROM pg_auth_members
    WHERE member::regrole = current_user::regrole
        AND roleid::regrole = username::regrole
        AND admin_option
  ) INTO was_granted;
  IF was_granted THEN
    EXECUTE format('REVOKE %I FROM %I', username, current_user);
  END IF;
END;
$proc$ LANGUAGE plpgsql;

CREATE PROCEDURE pg_temp.setup_user(username TEXT, suffix TEXT, description TEXT) AS $proc$
DECLARE
  app TEXT;
BEGIN
  RAISE NOTICE '---';
  RAISE NOTICE 'Setting up % user %', description, username;
  CALL pg_temp.make_user(username);
  FOREACH app in ARRAY ARRAY['admin', 'build_cache', 'build_scans', 'keycloak'] LOOP
    CALL pg_temp.grant_role_to_user(app || '_' || suffix, username);
  END LOOP;
  CALL pg_temp.revoke_user_from_current(username);
END;
$proc$ LANGUAGE plpgsql;

CREATE PROCEDURE pg_temp.setup_app_user(username TEXT) AS $proc$
BEGIN
  CALL pg_temp.setup_user(username, 'user', 'application');
END;
$proc$ LANGUAGE plpgsql;

CREATE PROCEDURE pg_temp.setup_migrator_user(username TEXT) AS $proc$
BEGIN
  CALL pg_temp.setup_user(username, 'owner', 'migrator');
END;
$proc$ LANGUAGE plpgsql;

CREATE PROCEDURE pg_temp.setup_monitor_user(username TEXT) AS $proc$
BEGIN
  RAISE NOTICE '---';
  RAISE NOTICE 'Setting up monitor user %', username;
  CALL pg_temp.make_user(username);
  CALL pg_temp.grant_role_to_user('pg_read_all_stats', username);
  CALL pg_temp.revoke_user_from_current(username);
END;
$proc$ LANGUAGE plpgsql;

DO $$
BEGIN
  CALL pg_temp.verify_required_roles();

  -- EDIT THE FOLLOWING LINES TO ALTER DB USERS SET UP
  CALL pg_temp.setup_app_user('ge_app');
  CALL pg_temp.setup_migrator_user('ge_migrator');
  CALL pg_temp.setup_monitor_user('ge_monitor');
  -- END - DO NOT EDIT PAST HERE

  COMMIT;
  RAISE NOTICE '---';
  RAISE NOTICE 'Database users set up successfully';
END;
$$ LANGUAGE plpgsql;
